@@ -38,6 +38,7 @@
 #' @param n_snp the number of (consecutive) markers to include in the scan
 #' @param max_iter maximum number of iterations for EM algorithm
 #' @param max_prec stepwise precision for EM algorithm. EM stops once incremental difference in log likelihood is less than max_prec
+#' @param cores number of cores to use when parallelizing via parallel::mclapply. Set to 1 for no parallelization.
 #' @export
 #' @importFrom stats var
 #' @references Knott SA, Haley CS (2000) Multitrait
@@ -70,7 +71,7 @@
 #' colnames(probs) <- LETTERS[1:2]
 #' dimnames(probs)[[3]] <- paste0("m", 1:5)
 #' scan_pvl(probs = probs, pheno = pheno, kinship = NULL,
-#' start_snp = 1, n_snp = 5)
+#' start_snp = 1, n_snp = 5, cores = 1)
 #' @importFrom rlang .data
 #' @return a tibble with d + 1 columns. First d columns indicate the genetic data (by listing the marker ids) used in the design matrix; last is log10 likelihood
 
@@ -81,108 +82,29 @@ scan_pvl <- function(probs,
                      start_snp = 1,
                      n_snp,
                      max_iter = 1e+04,
-                     max_prec = 1 / 1e+08
+                     max_prec = 1 / 1e+08,
+                     cores = 1
                      )
     {
-    if (is.null(probs)) stop("probs is NULL")
-    if (is.null(pheno)) stop("pheno is NULL")
-    stopifnot(!is.null(rownames(probs)),
-              !is.null(colnames(probs)),
-              !is.null(dimnames(probs)[[3]]),
-              !is.null(rownames(pheno)),
-              n_snp > 0,
-              start_snp > 0,
-              start_snp + n_snp - 1 <= dim(probs)[3]
-        )
-    # check additional conditions when addcovar is not NULL
-    if (!is.null(addcovar)) {
-        stopifnot(!is.null(rownames(addcovar)),
-                  !is.null(colnames(addcovar))
-                  )
-    }
-    d_size <- ncol(pheno)  # d_size is the number of univariate phenotypes
-    # force things to be matrices
-    if(!is.matrix(pheno)) {
-        pheno <- as.matrix(pheno)
-        if(!is.numeric(pheno)) stop("pheno is not numeric")
-    }
-    if(is.null(colnames(pheno))){ # force column names
-        colnames(pheno) <- paste0("pheno", seq_len(ncol(pheno)))}
-    if(!is.null(addcovar)) {
-        if(!is.matrix(addcovar)) addcovar <- as.matrix(addcovar)
-        if(!is.numeric(addcovar)) stop("addcovar is not numeric")
-    }
-
-    # find individuals in common across all arguments
-    # and drop individuals with missing covariates or missing *one or more* phenotypes
-    # need to consider presence or absence of different inputs: kinship, addcovar
-    id2keep <- make_id2keep(probs = probs,
-                            pheno = pheno,
-                            addcovar = addcovar,
-                            kinship = kinship
-                            )
-    # remove - from id2keep vector - subjects with a missing phenotype or covariate
-    pheno <- subset_input(input = pheno, id2keep = id2keep)
-    subjects_phe <- check_missingness(pheno)
-    id2keep <- intersect(id2keep, subjects_phe)
-
-    if (!is.null(addcovar)) {
-        addcovar <- subset_input(input = addcovar, id2keep = id2keep)
-        subjects_cov <- check_missingness(addcovar)
-        id2keep <- intersect(id2keep, subjects_cov)
-        addcovar <- subset_input(input = addcovar, id2keep = id2keep)
-    }
-    if (!is.null(addcovar)) {
-        addcovar <- drop_depcols(addcovar)
-    }
-    # Send messages if there are two or fewer subjects
-    if (length(id2keep) == 0){stop("no individuals common to all inputs")}
-    if (length(id2keep) <= 2){
-        stop(paste0("only ", length(id2keep),
-                    " common individual(s): ",
-                    paste(id2keep, collapse = ": ")))
-        }
-    # subset inputs to get all without missingness
-    probs <- subset_input(input = probs, id2keep = id2keep)
-    pheno <- subset_input(input = pheno, id2keep = id2keep)
-    if (d_size != Matrix::rankMatrix(pheno)) stop("Phenotypes matrix is not full rank. Input only full-rank phenotypes matrices.")
-
-    if (!is.null(kinship)) {
-        kinship <- subset_kinship(kinship = kinship, id2keep = id2keep)
-    }
-    if (!is.null(addcovar)) {
-        addcovar <- subset_input(input = addcovar, id2keep = id2keep)
-    }
-    if (!is.null(kinship)){
-        # covariance matrix estimation
-        message(paste0("starting covariance matrices estimation with data from ", length(id2keep), " subjects."))
-        # first, run gemma2::MphEM(), by way of calc_covs(), to get Vg and Ve
-        cc_out <- calc_covs(pheno, kinship, max_iter = max_iter, max_prec = max_prec, covariates = addcovar)
-        Vg <- cc_out$Vg
-        Ve <- cc_out$Ve
-        message("covariance matrices estimation completed.")
-        # define Sigma
-        Sigma <- calc_Sigma(Vg, Ve, kinship)
-    }
-    if (is.null(kinship)){
-        # get Sigma for Haley Knott regression without random effect
-        Ve <- var(pheno) # get d by d covar matrix
-        Sigma <- calc_Sigma(Vg = NULL, Ve = Ve, n_mouse = nrow(pheno))
-    }
-
-    # define Sigma_inv
-    Sigma_inv <- solve(Sigma)
+    inputs <- process_inputs(probs = probs,
+                             pheno = pheno,
+                             addcovar = addcovar,
+                             kinship = kinship,
+                             max_iter = max_iter,
+                             max_prec = max_prec)
     # prepare table of marker indices for each call of scan_pvl
+    d_size <- ncol(inputs$pheno)
     mytab <- prep_mytab(d_size = d_size, n_snp = n_snp)
     # set up parallel analysis
     out <- scan_pvl_clean(mytab = mytab,
-                          addcovar = addcovar,
-                          probs = probs,
-                          Sigma_inv = Sigma_inv,
-                          Sigma = Sigma,
+                          addcovar = inputs$addcovar,
+                          probs = inputs$probs,
+                          Sigma_inv = inputs$Sigma_inv,
+                          Sigma = inputs$Sigma,
                           start_snp = start_snp,
-                          pheno = pheno,
-                          n_snp = n_snp
+                          pheno = inputs$pheno,
+                          n_snp = n_snp,
+                          cores = cores
                           )
     return(out)
 }
@@ -196,19 +118,26 @@ scan_pvl_clean <- function(pheno,
                            Sigma,
                            start_snp,
                            mytab,
-                           n_snp){
-    list_result <- furrr::future_map(.x = as.data.frame(t(mytab)),
-                                     .f = fit1_pvl,
+                           n_snp,
+                           cores = 1){
+    list_result <- parallel::mclapply(X = as.data.frame(t(mytab)),
+                                     FUN = fit1_pvl,
                                      addcovar = addcovar,
                                      probs = probs,
                                      inv_S = Sigma_inv,
                                      S = Sigma,
                                      start_snp = start_snp,
-                                     pheno = pheno
+                                     pheno = pheno,
+                                     mc.cores = cores
                                      )
     mytab$loglik <- unlist(list_result)
     marker_id <- dimnames(probs)[[3]][start_snp:(start_snp + n_snp - 1)]
-    mytab2 <- tibble::as_tibble(apply(FUN = function(x) marker_id[x], X = mytab[, -ncol(mytab)], MARGIN = 2))
+    mytab2 <- tibble::as_tibble(apply(FUN = function(x) marker_id[x],
+                                      X = mytab[, -ncol(mytab)],
+                                      MARGIN = 2))
     mytab2$log10lik <- mytab$loglik / log(10)
     return(mytab2)
 }
+
+
+
